/*************************************************************************************************
Copyright (c) 2000 BindView Corporation.  All rights reserved.  

By using this software, YOU AGREE to the following license terms.  IF YOU DO NOT AGREE, YOU MAY NOT USE THE SOFTWARE.  
1.  BindView believes that this software is safe for use in normal circumstances, and has performed what it believes to be reasonable but non-exhaustive testing to verify this.  The software is intended for use only by experienced and knowledgeable computer professionals; IT IS PROVIDED "AS IS, WITH ALL FAULTS," including source code so that the user can study the source code and independently determine the software's suitability.  BindView makes no warranty of any kind, express or implied, and DISCLAIMS ANY AND ALL WARRANTIES, CONDITIONS, OR IMPLIED TERM OF QUALITY, INCLUDING THE IMPLIED WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE.  All use of the software is entirely at the user's own risk.
2.  IN NO EVENT WILL BINDVIEW BE LIABLE FOR DAMAGES OF ANY KIND arising from or relating to use of the software, whether such damages are direct, indirect, incidental, consequential, exemplary, or any other kind, and whether arising under contract, tort (including negligence), strict liability, or otherwise.
3.  BindView will not object to your distribution of complete, unmodified copies of the distribution package of the software as provided by BindView, PROVIDED that you do not charge a fee other than a reasonable fee for distribution services.  You may charge a fee for any warranty or support services that you offer to purchasers of copies of the software.
4.  You may modify the software and distribute copies of the modified software, PROVIDED:
	(a) that you distribute, together with the executable code of the modified software:
		(1) the source code of the modified software, which must contain the BindView copyright notice set forth above (in addition to your own copyright notice if any); and
		(2) a copy of the complete, unmodified distribution package of the software as provided by BindView; and
	(b) that you clearly indicate in the source code and in an accompanying documentation file that the software is based on BindView's software and was modified by you; and
	(c) that you grant users of the modified software the same rights as are granted to you by this license.
**************************************************************************************************/
#include <winsock2.h>
#include "icmp.h"

/*******************************************************************/
// PING.C
/*******************************************************************/

#pragma pack(4) 
 
#define ICMP_ECHO 8 
#define ICMP_ECHOREPLY 0 
 
#define ICMP_MIN 8 // minimum 8 byte icmp packet (just header) 
 
struct Udp
{
	USHORT udpSourcePortNumber;
	USHORT udpDestinationPortNumber;
	USHORT udpLength;
	USHORT udpChecksum;
};



/* ICMP header */
typedef struct _ihdr { 
  BYTE i_type; 
  BYTE i_code; /* type sub code */ 
  USHORT i_cksum; 
  USHORT i_id; 
  USHORT i_seq; 
  /* This is not the std header, but we reserve space for time */ 
  //ULONG timestamp; 
} IcmpHeader; 
static USHORT checksum(USHORT *, int); 
#define MAX_PACKET_SIZE 1500
#define DEFAULT_TIMEOUT 100


bool ping(unsigned int dest_addr,unsigned char *data,
          int data_len,
          USHORT packet_id  )
////////////////////////////////////////////////////////////////////
{ 
  char ICMP_DATA[MAX_PACKET_SIZE]; 
  char *icmp_data=ICMP_DATA;
  int datasize=data_len;
  int rc;

   // creates a socket that is bound to Control Message Protocol 
  SOCKET sockRaw;
  sockRaw = WSASocket(AF_INET, SOCK_RAW, IPPROTO_ICMP, NULL, 0,0); 
  if (sockRaw == INVALID_SOCKET) 
	  return false;

  int timeout;
  
  // sets send timeout
  timeout = DEFAULT_TIMEOUT; 
  rc = setsockopt(sockRaw, SOL_SOCKET,SO_RCVTIMEO,
                    (char*)&timeout, sizeof(timeout)); 
  if(rc == SOCKET_ERROR) 
	  return false;

  struct sockaddr_in dest;
  // resolves destination host 
  dest.sin_addr.s_addr = dest_addr; 
  dest.sin_family = AF_INET; 

  char *dest_ip; 
  dest_ip = inet_ntoa(dest.sin_addr); 

  // alloc icmp data
  data_len += sizeof(IcmpHeader);   
 
  // Fill in various stuff in our ICMP request.
  memset(icmp_data, 0, MAX_PACKET_SIZE); 
 
  IcmpHeader *icmp_hdr; 
  char *datapart; 
 
  icmp_hdr = (IcmpHeader*)icmp_data; 
 
  icmp_hdr->i_type = ICMP_ECHOREPLY; 
  icmp_hdr->i_code = 0; 
  icmp_hdr->i_id = (USHORT)packet_id; 
  icmp_hdr->i_cksum = 0; 
  icmp_hdr->i_seq = 0; 
   
  datapart = icmp_data + sizeof(IcmpHeader); 
  // Place data in the buffer. 
  memcpy(datapart,data,datasize); 

  static USHORT seq_no = 0; 
  ((IcmpHeader*)icmp_data)->i_cksum = 0; 
  ((IcmpHeader*)icmp_data)->i_seq = seq_no++; 
  ((IcmpHeader*)icmp_data)->i_cksum = 
       checksum((USHORT*)icmp_data, data_len); 
 
  if (sendto(sockRaw, icmp_data, data_len, 0,
	  (struct sockaddr*)&dest, sizeof(dest)) == SOCKET_ERROR) {
			  return false;
  } 
     
  if (closesocket ( sockRaw ) == SOCKET_ERROR) return false;

  return true;
} 
//////////////////////////////////////////////////////////////////// 
USHORT  checksum(USHORT *buffer, int size) 
//////////////////////////////////////////////////////////////////// 
{
  unsigned long cksum=0; 
 
  while(size >1) { 
    cksum+=*buffer++; 
    size -=sizeof(USHORT); 
  } 
   
  if(size ) { 
    cksum += *(UCHAR*)buffer; 
  } 
 
  cksum = (cksum >> 16) + (cksum & 0xffff); 
  cksum += (cksum >>16); 
  return (USHORT)(~cksum); 
} 

